# -*- coding: utf-8 -*-
from contenttypes.basic import _
from contenttypes.basic.fields.osmap import OSMapField
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IOSMapBehaviorSchema(model.Schema):
    """ OSmap Behavior Schema
    """

    geolocation = OSMapField(
        title=_('Geolocation'),
        description=_('Enter an address or set the location mark on the map'),
        required=False,
    )


@implementer(IOSMapBehaviorSchema)
@adapter(IDexterityContent)
class OSMapBehaviorFactory(object):

    def __init__(self, context):
        self.context = context

    @property
    def geolocation(self):
        return self.context.geolocation

    @geolocation.setter
    def geolocation(self, value):
        self.context.geolocation = value



# -*- coding: utf-8 -*-
from contenttypes.basic.testing import CONTENTTYPES_BASIC_CONTROLPANEL_TESTING
from contenttypes.basic.tests import TestUtils
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID


class ControlPanelFunctionalTest(TestUtils):

    layer = CONTENTTYPES_BASIC_CONTROLPANEL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.portal_url = self.portal.absolute_url()
        self.portal_setup = api.portal.get_tool('portal_setup')
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_coordinates_controlpanel(self):
        self._set_browser()
        self.browser.open(self.portal_url + '/@@coordinates-controlpanel')
        contents = self.browser.contents
        field = 'form-widgets-map_center'
        self.assertTrue(field in contents)



# -*- coding: utf-8 -*-
from contenttypes.basic import _
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


PositionVocabulary = SimpleVocabulary([
    SimpleTerm(value=1, title=_('Upper - Left')),
    SimpleTerm(value=2, title=_('Upper - Centre')),
    SimpleTerm(value=3, title=_('Upper - Right')),
    SimpleTerm(value=4, title=_('Centre - Left')),
    SimpleTerm(value=5, title=_('Centre - Centre')),
    SimpleTerm(value=6, title=_('Centre - Right')),
    SimpleTerm(value=7, title=_('Bottom - Left')),
    SimpleTerm(value=8, title=_('Bottom - Centre')),
    SimpleTerm(value=9, title=_('Bottom - Right')),
])


@provider(IFormFieldProvider)
class ITextPositionSchema(model.Schema):
    """ Text Position Schema
    """

    text_position = schema.Choice(
        title=_('Position'),
        description=_(
            'Position where the title and the button will be displayed over '
            'the image. Only useful for medium and up screens.',
        ),
        vocabulary=PositionVocabulary,
        required=False,
        default=5,
    )


@implementer(ITextPositionSchema)
@adapter(IDexterityContent)
class TextPositionFactory(object):

    def __init__(self, context):
        self.context = context

    @property
    def text_position(self):
        return self.context.text_position

    @text_position.setter
    def text_position(self, value):
        self.context.text_position = value



# -*- coding: utf-8 -*-
from contenttypes.basic.interfaces import IOSMapField
from zope.interface import implementer
from zope.schema import TextLine


@implementer(IOSMapField)
class OSMapField(TextLine):
    """ OSMap Field
    """



# -*- coding: utf-8 -*-
from contenttypes.basic.interfaces import IAnnouncement
from contenttypes.basic.interfaces import IBanner
from contenttypes.basic.interfaces import IIcon
from contenttypes.basic.interfaces import ISlide
from plone.dexterity.content import Container
from zope.interface import implementer


@implementer(IAnnouncement)
class Announcement(Container):
    """ Convenience subclass for ``Announcement`` portal type
    """


@implementer(IBanner)
class Banner(Container):
    """ Convenience subclass for ``Banner`` portal type
    """


@implementer(IIcon)
class Icon(Container):
    """ Convenience subclass for ``Icon`` portal type
    """


@implementer(ISlide)
class Slide(Container):
    """ Convenience subclass for ``Slide`` portal type
    """



# -*- coding: utf-8 -*-
from contenttypes.basic.interfaces import IOSMapAdvancedField
from zope.interface import implementer
from zope.schema import TextLine


@implementer(IOSMapAdvancedField)
class OSMapAdvancedField(TextLine):
    """ OSMap Advanced Field
    """



# -*- coding: utf-8 -*-
from contenttypes.basic.testing import CONTENTTYPES_BASIC_FUNCTIONAL_TESTING
from contenttypes.basic.tests import TestUtils
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID


class ConvertersFunctionalTest(TestUtils):

    layer = CONTENTTYPES_BASIC_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_osmap_converter(self):
        self._create_dummy_fti(behavior='contenttypes.basic.osmap')
        self._create_obj()
        self._set_browser()

        self.browser.open(self.portal_url + '/test/edit')
        geolocation = self.browser.getControl(
            name='form.widgets.IOSMapBehaviorSchema.geolocation',
        )
        geolocation_value = 'wrong_number|0.6267842'
        geolocation.value = geolocation_value
        save = self.browser.getControl(name='form.buttons.save')
        save.click()
        self.assertTrue(self.obj.geolocation, '0|0.6267842')

        self.browser.open(self.portal_url + '/test/edit')
        geolocation = self.browser.getControl(
            name='form.widgets.IOSMapBehaviorSchema.geolocation',
        )
        geolocation_value = 'wrong_number|wrong_number'
        geolocation.value = geolocation_value
        save = self.browser.getControl(name='form.buttons.save')
        save.click()
        self.assertTrue(self.obj.geolocation, '0|0')

        self.browser.open(self.portal_url + '/test/edit')
        geolocation = self.browser.getControl(
            name='form.widgets.IOSMapBehaviorSchema.geolocation',
        )
        geolocation_value = 'wrong_length'
        geolocation.value = geolocation_value
        save = self.browser.getControl(name='form.buttons.save')
        save.click()
        self.assertTrue(self.obj.geolocation, '0|0')

        self.browser.open(self.portal_url + '/test/edit')
        geolocation = self.browser.getControl(
            name='form.widgets.IOSMapBehaviorSchema.geolocation',
        )
        geolocation_value = '0.1|0.1'
        geolocation.value = geolocation_value
        save = self.browser.getControl(name='form.buttons.save')
        save.click()
        self.assertTrue(self.obj.geolocation, '0.1|0.1')



# -*- coding: utf-8 -*-
from contenttypes.basic import _
from contenttypes.basic.interfaces import IOSMapAdvancedField
from contenttypes.basic.interfaces import IOSMapAdvancedWidget
from datetime import datetime
from plone import api
from plone.api.exc import InvalidParameterError
from z3c.form.browser.text import TextWidget
from z3c.form.converter import BaseDataConverter
from z3c.form.interfaces import IFieldWidget
from z3c.form.interfaces import IFormLayer
from z3c.form.widget import FieldWidget
from zope.component import adapter
from zope.interface import implementer

import json


@implementer(IOSMapAdvancedWidget)
class OSMapAdvancedWidget(TextWidget):

    default_point_value = '{"type": "point","coordinates": [[0,0]]}'

    name = 'osmap-widget'
    label = _('OSMap Widget')
    timestamp = datetime.now().strftime('%s')

    @property
    def portal_url(self):
        return api.portal.get().absolute_url()

    @property
    def default_value(self):
        return self.field.default or self.default_point_value

    def default_value_map(self):
        try:
            default_coords = api.portal.get_registry_record(name='coordinates.map_center')
            default_value = """{
                "type": "point",
                "coordinates": [[""" + default_coords.replace('|', ',') + """]]
            }"""
            return default_value
        except InvalidParameterError:
            return self.default_point_value


@implementer(IFieldWidget)
@adapter(IOSMapAdvancedField, IFormLayer)
def OSMapAdvancedFieldWidget(field, request):
    return FieldWidget(field, OSMapAdvancedWidget(request))


@adapter(IOSMapAdvancedField, IOSMapAdvancedWidget)
class OSMapAdvancedConverter(BaseDataConverter):
    """ Convert between the context and the widget
    """

    def toWidgetValue(self, value):
        return self.parse_coordinates(value)

    def toFieldValue(self, value):
        return self.parse_coordinates(value)

    def parse_coordinates(self, value):
        if not value:
            return ''

        try:
            json.loads(value)
        except json.decoder.JSONDecodeError:
            return ''

        return value



# -*- coding: utf-8 -*-
from contenttypes.basic import _
from contenttypes.basic.fields.osmap_advanced import OSMapAdvancedField
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IOSMapAdvancedBehaviorSchema(model.Schema):
    """ OSmap Advanced Behavior Schema
    """

    advanced_geolocation = OSMapAdvancedField(
        title=_('Geolocation'),
        description=_('Enter an address or set the location mark on the map'),
        required=False,
    )


@implementer(IOSMapAdvancedBehaviorSchema)
@adapter(IDexterityContent)
class OSMapAdvancedBehaviorFactory(object):

    def __init__(self, context):
        self.context = context

    @property
    def advanced_geolocation(self):
        return self.context.advanced_geolocation

    @advanced_geolocation.setter
    def advanced_geolocation(self, value):
        self.context.advanced_geolocation = value



# -*- coding: utf-8 -*-
from contenttypes.basic import _
from plone import api
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider
from zope.schema.interfaces import IContextAwareDefaultFactory


MORE_INFORMATION = _('More information')


@provider(IContextAwareDefaultFactory)
def default_button_text(context):
    return api.portal.translate('More information', 'contenttypes.basic', lang=api.portal.get_current_language())


@provider(IFormFieldProvider)
class IButtonTextSchema(model.Schema):
    """ Button Text Schema
    """

    button_text = schema.TextLine(
        title=_(u'Button text'),
        required=False,
        defaultFactory=default_button_text,
    )


@implementer(IButtonTextSchema)
@adapter(IDexterityContent)
class ButtonTextFactory(object):

    def __init__(self, context):
        self.context = context

    @property
    def button_text(self):
        return self.context.button_text

    @button_text.setter
    def button_text(self, value):
        self.context.button_text = value



# -*- coding: utf-8 -*-
from contenttypes.basic import _
from contenttypes.basic.interfaces import IColourField
from contenttypes.basic.interfaces import IColourWidget
from z3c.form.browser.text import TextWidget
from z3c.form.interfaces import IFieldWidget
from z3c.form.interfaces import IFormLayer
from z3c.form.widget import FieldWidget
from zope.component import adapter
from zope.interface import implementer


@implementer(IColourWidget)
class ColourWidget(TextWidget):

    name = 'colour-widget'
    label = _('Colour Widget')


@implementer(IFieldWidget)
@adapter(IColourField, IFormLayer)
def ColourFieldWidget(field, request):
    return FieldWidget(field, ColourWidget(request))
